/* eslint-disable @typescript-eslint/no-useless-constructor */

import React from "react"
import * as FitUi from "fit-ui"
const Fit: typeof FitUi = window.Fit;

type ControlProps = {
    dataType?: "Date" | "String" | "Color"
}

type ControlState = {
    value: string
}

export default class GenericControl extends React.Component<ControlProps, ControlState> {
    private domRef = React.createRef<HTMLDivElement>();
    private control: Fit.Controls.ControlBase | undefined;

    // Control wrapper does not support React state so we avoid
    // using it: https://github.com/bryntum/support/issues/6912
    private myState: ControlState = {
        value: ""
    }

    constructor(props: ControlProps) {
        // Feature request:
        // Tell Gantt to NOT auto commit when cell lose focus or when
        // pressing ENTER, and don't exit/cancel when pressing ESC either.
        //this.manuallyCommit(true)

        super(props);
    }

    render() {
        if (this.control) {
            this.control.Value(this.myState.value)
        }

        return <div style={{ padding: "0.6em", color: "black", width: "100%" }} ref={this.domRef}></div>;
    }

    componentDidMount() {
        if (!this.control) {
            if (this.props.dataType === "Date") {
                this.control = new Fit.Controls.DatePicker();

                // Hack to make date picker control work in Bryntum Gantt.
                // Normally, the calendar widget is mounted in the document root.
                // But focus is removed from the grid cell when interacting with
                // a widget not mounted in the cell, which causes it to exit editing
                // mode, which in turn breaks the functionality.
                // With a more refined controls API this should no longer be necessary,
                // as the control will be able to decide when to exit editing mode.
                // The line below makes the calendar widget mount in the grid cell.
                // Furthermore a CSS hack has been added to index.html with some styling
                // that allows for the calendar widget to exceed the boundaries of the grid cell.
                (this.control as any)._internal.ReduceDocumentRootPollution = true
            } else if (this.props.dataType === "Color") {
                const dropdown = new Fit.Controls.DropDown();
                const listview = new Fit.Controls.ListView();
                
                listview.AddItem("&nbsp;", "");
                ["teal", "blue", "orange", "green", "purple"].forEach((color) => {
                    listview.AddItem(`<span style='color: ${color}'>${color}</span>`, color);
                });

                dropdown.TextSelectionMode(true);
                dropdown.MultiSelectionMode(false);
                dropdown.DropDownMaxHeight(20, "em");
                dropdown.SetPicker(listview);
                
                this.control = dropdown;
            } else {
                this.control = new Fit.Controls.Input();
            }

            this.control.Width(100, "%");

            this.control.OnChange((sender) => {
                if (sender instanceof Fit.Controls.DropDown) {
                    this.myState.value = sender.GetSelections()[0]?.Value || "";
                } else {
                    this.myState.value = sender.Value()
                }

                if ((sender instanceof Fit.Controls.Input) === false) {
                    this.exitEditingMode();
                }
            });
        }

        this.control.Value(this.myState.value)
        this.domRef.current && this.control.Render(this.domRef.current);
    }

    componentWillUnmount() {
        this.control?.Dispose();
    }

    private exitEditingMode() {
        if (this.control?.IsValid()) {
            // Feature request: Make gantt exit editing mode and call getValue() to persist change
            // this.commitChange()
            // Work around:
            this.control.Focused(false);
        }
    }

    // Bryntum cell editing interface - required functions:

    getValue() {
        if (this.props.dataType === "Date") {
            return new Date(this.myState.value);
        }

        return this.myState.value
    }

    setValue(value: string | number | Date | undefined | null, cellEditorContext?: { columnId: string; columnIndex: number; rowIndex: number }) {
        // Control wrapper does not support React state so we avoid
        // using it: https://github.com/bryntum/support/issues/6912
        this.myState = {
            value: value ? value + "" : ""
        }

        if (value instanceof Date) {
            this.myState.value = Fit.Date.Format(value, "YYYY-MM-DD")
        }

        // Force re-render with updated fake state
        this.forceUpdate()
    }

    isValid() {
        // Gantt can be configured to prevent editor from closing when value is invalid
        return this.control?.IsValid() || false
    }

    focus() {
        // IMPORTANT: If focus is not assigned to an element within cell, then gantt lose the
        // ability to exit editing mode. An element must have focus so it can lose focus to exit edit mode.
        // Element must retain focus on every render for gantt to work properly and be able to exit edit mode.
        // Gantt will call focus() initially and if exiting edit mode is prevented via beforeCancel or beforeComplete.

        this.control?.Focused(true);

        // Open control on focus
        if (this.control instanceof Fit.Controls.DropDown) {
            // Using setTimeout to avoid auto closing. DropDown closes when
            // clicking outside of DropDown, and unfortunately onclick in
            // grid cell fires after control is rendered and opened.
            setTimeout(this.control.OpenDropDown);
        } else if (this.control instanceof Fit.Controls.DatePicker) {
            this.control.Show();
        }
    }
}

// Preload resources for controls - CDN is sometimes extremly slow (watch Network tab in Dev Tools)

var dp = new Fit.Controls.DatePicker();
dp.GetDomElement().style.visibility = "hidden";
dp.Render(document.body);
dp.Show();
dp.Dispose();

var dd = new Fit.Controls.DropDown();
dd.GetDomElement().style.visibility = "hidden";
dd.Render(document.body);
setTimeout(dd.Dispose);